﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

using Microsoft.Devices.Sensors;

namespace SilverPong
{
    public partial class GamePage : PhoneApplicationPage
    {
        ContentManager contentManager;
        GameTimer timer;
        SpriteBatch spriteBatch;

        // For rendering the XAML onto a texture
        UIElementRenderer elementRenderer;

        // Game World

        Texture2D ballTexture;
        Microsoft.Xna.Framework.Rectangle ballRectangle;
        float ballX;
        float ballY;
        float ballXSpeed = 3;
        float ballYSpeed = 3;

        Texture2D lPaddleTexture;
        Microsoft.Xna.Framework.Rectangle lPaddleRectangle;
        float lPaddleSpeed = 6;
        float lPaddleY;
        int lPlayerScore;

        Texture2D rPaddleTexture;
        Microsoft.Xna.Framework.Rectangle rPaddleRectangle;
        float rPaddleY;
        int rPlayerScore;

        // Distance of paddles from screen edge
        int margin;

        Vector3 accelState = Vector3.Zero;

        void acc_ReadingChanged
                      (object sender, AccelerometerReadingEventArgs e)
        {
            accelState.X = (float)e.X;
            accelState.Y = (float)e.Y;
            accelState.Z = (float)e.Z;
        }

        Accelerometer acc;

        public GamePage()
        {
            InitializeComponent();

            // Get the content manager from the application
            contentManager = (Application.Current as App).Content;

            // Create a timer for this page
            timer = new GameTimer();
            timer.UpdateInterval = TimeSpan.FromTicks(333333);
            timer.Update += OnUpdate;
            timer.Draw += OnDraw;

            acc = new Accelerometer();
            acc.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<AccelerometerReading>>(acc_CurrentValueChanged);

        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Set the sharing mode of the graphics device to turn on XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(true);

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(SharedGraphicsDeviceManager.Current.GraphicsDevice);

            ballTexture = contentManager.Load<Texture2D>("ball");
            lPaddleTexture = contentManager.Load<Texture2D>("lpaddle");
            rPaddleTexture = contentManager.Load<Texture2D>("rpaddle");

            ballRectangle = new Microsoft.Xna.Framework.Rectangle(
                0, 0,
                (int)this.ActualWidth / 20,
                (int)this.ActualWidth / 20);

            ballX = (int)this.ActualWidth / 2;
            ballY = (int)this.ActualHeight / 2;

            margin = (int)this.ActualWidth / 20;

            lPaddleRectangle = new Microsoft.Xna.Framework.Rectangle(
                margin, 0,
                (int)this.ActualWidth / 20,
                (int)this.ActualHeight / 5);

            rPaddleRectangle = new Microsoft.Xna.Framework.Rectangle(
                (int)this.ActualWidth - lPaddleRectangle.Width - margin, 0,
                (int)this.ActualWidth / 20,
                (int)this.ActualHeight / 5);

            lPaddleY = lPaddleRectangle.Y;
            rPaddleY = rPaddleRectangle.Y;

            acc.Start();

            // Start the timer
            timer.Start();

            // Create the element renderer
            if (null == elementRenderer)
            {
                elementRenderer = new UIElementRenderer(this, (int)ActualWidth, (int)ActualHeight);
            }

            base.OnNavigatedTo(e);
        }

        void acc_CurrentValueChanged(object sender, SensorReadingEventArgs<AccelerometerReading> e)
        {
            accelState.X = (float)e.SensorReading.Acceleration.X;
            accelState.Y = (float)e.SensorReading.Acceleration.Y;
            accelState.Z = (float)e.SensorReading.Acceleration.Z;
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            // Stop the timer
            timer.Stop();

            acc.Stop();

            // Set the sharing mode of the graphics device to turn off XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(false);

            base.OnNavigatedFrom(e);
        }

        /// <summary>
        /// Allows the page to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        private void OnUpdate(object sender, GameTimerEventArgs e)
        {
            ballX = ballX + ballXSpeed;
            ballY = ballY + ballYSpeed;

            if (ballX < 0)
            {
                rPlayerScore++;
                ScoreTextBlock.Text = lPlayerScore.ToString() + ":" + rPlayerScore.ToString();
                ballXSpeed = -ballXSpeed;
            }

            if (ballX + ballRectangle.Width > this.ActualWidth)
            {
                lPlayerScore++;
                ScoreTextBlock.Text = lPlayerScore.ToString() + ":" + rPlayerScore.ToString();
                ballXSpeed = -ballXSpeed;
            }

            if (ballY < 0 || ballY + ballRectangle.Height > this.ActualHeight)
            {
                ballYSpeed = -ballYSpeed;
            }

            ballRectangle.X = (int)(ballX + 0.5f);
            ballRectangle.Y = (int)(ballY + 0.5f);

            lPaddleY = lPaddleY - (accelState.X * lPaddleSpeed);

            lPaddleRectangle.Y = (int)(lPaddleY + 0.5f);

            // Make the right paddle always follow the ball

            rPaddleRectangle.Y = ballRectangle.Y;

            // Ball - Bat collisons

            if (ballRectangle.Intersects(lPaddleRectangle))
            {
                ballXSpeed = -ballXSpeed;
            }

            if (ballRectangle.Intersects(rPaddleRectangle))
            {
                ballXSpeed = -ballXSpeed;
            }
        }

        /// <summary>
        /// Allows the page to draw itself.
        /// </summary>
        private void OnDraw(object sender, GameTimerEventArgs e)
        {

            // Render the Silverlight controls using the UIElementRenderer.
            elementRenderer.Render();

            SharedGraphicsDeviceManager.Current.GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            spriteBatch.Draw(ballTexture, ballRectangle, Color.White);
            spriteBatch.Draw(lPaddleTexture, lPaddleRectangle, Color.White);
            spriteBatch.Draw(rPaddleTexture, rPaddleRectangle, Color.White);

            spriteBatch.Draw(elementRenderer.Texture, Vector2.Zero, Color.White);

            spriteBatch.End();
        }

        private void quitButton_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
    }
}